/**
 * Augment the typings of Vue.js
 */

import Vue from 'vue';
import { Store } from 'vuex';
import 'vue-meta';
import { RendererInterface, RendererOption } from '@riejs/rie';

/**
 * asyncData 参数
 */
interface IAsyncDataOption {
  /** 如果组件传入了 getStore ，store 就是 getStore 的返回结果，否则为 undefined */
  store: Store;
};
type AsyncData<T> = (asyncDataOption: IAsyncDataOption) => Promise<T>;
type StoreFactory<T> = () => Store<T>;

declare module 'vue/types/options' {
  interface ComponentOptions<V extends Vue> {
    /**
     * Rie 的扩展方法，用于渲染前获取异步数据。并将数据与组件本身的 data 进行混合
     */
    asyncData?: AsyncData;
    /**
     * Rie 的扩展方法，store 工厂函数，用于获取 store
     */
    getStore?: StoreFactory;
  }
}

declare class Renderer implements RendererInterface {
  public static type: string;
  public static packer: string;
  public option: RendererOption;
  public render: () => Promise<string>;
}

export { Vue, Renderer };
